var path = require('path');
var atomicBuilder = {};
var find = require('findit');
var Promise = require('promise');
var fs = require('fs');
var process_path = process.cwd();
var parser = require("./parser.js");
var compiler = require("./compiler.js");
var mkdirp = require('mkdirp');
var getDirName = path.dirname;

var getFileInfo = function(dir){
  return new Promise(function(resolve,reject){
       var finder = find(dir);
       var files = [];
       finder.on('file',function(path,stat){
            files.push(path);
       });
       finder.on('end',function() {
            resolve(files);
       });
  });
};

var isGreaterThan = function(a,b){
	var material = {
		atom:0,
		molecule:1,
		organism:2,
		template:3
	};
	if(material[a] > material[b]){
		return true;
	}else{
		return false;
	}
}

var makeAtomicArray = function(files,markup,styling){
	var components = [];
	var id = 0;
	for(var i = 0,n = files.length; i < n; i++){
		var file = files[i];
		var paths = file.split("/");
		var length = paths.length
		var category = paths[length - 3];
		var name = paths[length - 2];
		var extName = path.extname(file).replace(".","");
		var styleFile = name + '.' + styling;
		var markupFile = name + '.' + markup;
		var stylePath = file.replace(markupFile,styleFile);
		var markupPath = file.replace(markupFile,"readme.md");
		var baseName = path.basename(file,"."+markup);
		if(extName == markup && name == baseName){
        	var html = fs.readFileSync(file,'utf8');
        	var css = fs.readFileSync(stylePath,'utf8');
        	var about = fs.readFileSync(markupPath,'utf8');
        	id++;
        	components.push({category:category,name:name,html:html,css:css,about:about,id:id});
		}
	}
	return components;
}

var render = function(self,components,markup,styling){
	//textからpreview取得
	var text = self.html;
	var preview = parser.getPreview(text);
	preview = compiler.markup[markup](preview);
	//previewからコメント文取得
	var imports = "";
	// テンプレート取得
	while(1){
		var comment = parser.getComment(preview);
		if(!comment){
			break;
		}
		//commentからコンポーネント名取得
		var name = parser.getComponentName(comment);
		var flag = false;
		for(var i = 0,n = components.length; i < n; i++){
			var comp = components[i];
			if(name == comp.name){
				flag = true;
				//例えば、atomはmoleculeをincludeできない
				imports += parser.getImports(comp.html);
				if(self.id !== comp.id && !isGreaterThan(self.category,comp.category)){
					preview = preview.replace(comment,"");
					break;
				}
				// importされてなければ使えない
				if(self.id !== comp.id && imports.indexOf(name) == -1){
					preview = preview.replace(comment,"");
					break;
				}
				var template = parser.getTemplate(comp.html);
				var html = parser.getInnerHtmlFromTemplate(template);
				//templateに自身が含まれていたら削除(無限ループ回避)
				html = parser.removeSelf(html,comp.name);
				var defs = parser.getVarsFromTemplate(template);
				var overrides = parser.getVars(comment);
				html = parser.getRendered(html,defs,overrides);
				preview = preview.replace(comment,compiler.markup[markup](html));
				break;
			}
		}
		if(!flag){
			preview = preview.replace(comment,"");
		}
	}
	//スタイルシート取得
	var style = "";
	for(var i = 0,n = components.length; i < n; i++){
		var comp = components[i];
		if(imports.indexOf(comp.name) !== -1 || self.id == comp.id){
			style += comp.css;
		}
	}
	preview += "<style>"+compiler.styling[styling](style)+"</style>";
	//todo delete
	return parser.removeScript(preview);
}

var writeFile = function(path, contents, cb) {
  mkdirp(getDirName(path), function (err) {
    if (err) return cb(err);

    fs.writeFile(path, contents, cb);
  });
}

atomicBuilder.build = function(opt){
	var src = opt.src;
	var dist = opt.dist;
	var markup = opt.markup;
	var styling = opt.styling;
	return getFileInfo(path.resolve(process_path,src))
	.then(function(files){
		var arr = [];
		var components = makeAtomicArray(files,markup,styling);
		components.forEach(function(comp){
			var p = new Promise(function(resolve,reject){
				var html = render(comp,components,markup,styling);
				var target = dist+comp.category+"/"+comp.name+".html";
				writeFile(path.resolve(process_path,target),html,function(err){
					console.log(err);
					resolve();
				});
			});
			arr.push(p);
		});
		var json = JSON.stringify({components:components});
		var pjson = new Promise(function(resolve,reject){
			writeFile(path.resolve(process_path,dist+"/setting.json"),json,function(err){
				console.log(err);
				resolve();
			});
		});
		arr.push(pjson);
		return Promise.all(arr);
	});
}

module.exports = atomicBuilder;